import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTopology

sys.setrecursionlimit(10000)

def logistic(t, t0, tau0, L):

	return L / (1 + np.exp(-(t - t0) / tau0))

def logistic2(t, t0, tau0, L, t1, tau1):

	return L / (1 + np.exp( -1 * (((t - t0) / tau0) + ((t - t1) / tau1)) ))

def logistic3(t, t0, tau0, L, eps):
	print( t0, tau0, L, eps)
	return L / (1 + np.exp(-(t - t0) / tau0))**(1 + eps)

def recursivelyAddNodesToCluster(index, cluster, remaining, nConnections):

	# Have we been here?
	if(index not in cluster):

		# Add to cluster
		cluster.add(index)

		# Remove from global list
		remaining.remove(index)

		# Get next node
		for nIndex in nConnections[index]:
			cluster, remaining = recursivelyAddNodesToCluster(nIndex, cluster, remaining, nConnections)

	return cluster, remaining

def expFit(t, tau, y0, yf):

	return (yf - y0) * (1 - np.exp(-t / tau)) + y0

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]

vfs = [0.05,0.07,0.1]

modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]


vfs = [0.05]

numRuns = 2

# Figures
traceFig = plt.figure(figsize=(1.693,1.693))
timeFigT0 = plt.figure(figsize=(1.693,1.693))
timeFigTau = plt.figure(figsize=(1.693,1.693))

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		linkerDl = lOn2R * 2 * radius
		linkerK = kbt / np.power(linkerDl, 2.0)
		Lc = (monomersPerPolymer - 1) * (2 * radius + linkerL + linkerDl) + 2 * radius
		
		mIndex = 0

		t0s = []
		t0Errs = []

		taus = []
		tauErrs = []
		for model in models:

			# Time trace (every 50 frames)
			stepsize = 20
			
			t0Run = []
			t0RunErr = []

			tauRun = []
			tauRunErr = []
			for nr in range(numRuns):
			
				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
	
				tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				top = BioNetTopology.BioNetTopology()
				top.load(tpFname, rate=stepsize)

				time = []
				clusterTrace = []
				clusterErrTrace = []

				# Get fractal dimensions
				for frameIndex in range(top.numFrames):

					# Connect all the nodes
					top.frame[frameIndex].buildNodeConnections(top.numNodes)

					# Get a set of all node indices
					nodesRemaining = set(range(top.numNodes))

					# Initialise clusters
					clusters = []

					# Search while there are still nodes unaccounted for
					while(len(nodesRemaining) != 0):

						# New cluster
						cluster = set([])

						# Get a central node
						centralNode = nodesRemaining.pop()

						# Readd for recursion
						nodesRemaining.add(centralNode)

						# Recursively add
						cluster, nodesRemaining = recursivelyAddNodesToCluster(centralNode, cluster, nodesRemaining, top.frame[frameIndex].nConnections)
						
						# Add to clusters
						clusters.append(cluster)

					# Info
					numClusters = len(clusters)
					lengths = [len(c) for c in clusters]
					minCluster = min(lengths)
					maxCluster = max(lengths)

					time.append(top.frame[frameIndex].time)
					clusterTrace.append(maxCluster)

				# Plot
				time = np.array(time) / 1000.0
				plt.figure(traceFig.number)
				if(nr == 0):
					plt.plot(time, clusterTrace, "xC%d" % (mIndex), markersize=3, linewidth=1, label=r"M$_%d$" % (mIndex))
				else:
					plt.plot(time, clusterTrace, "xC%d" % (mIndex), markersize=3, linewidth=1)

				# Fit
				func = logistic
				guess = [10,1, numNodes]
				bounds = [[0,0,0],[np.inf,np.inf, numNodes]]
	#			bounds = [[0,0,0.99*numNodes],[np.inf,np.inf, 1.01 * numNodes]]

				popt, pcov = curve_fit(func, time, clusterTrace, guess, bounds=bounds)
				plt.plot(time, func(time, *popt), "--C%d" % (mIndex))

#				t0s.append(popt[0])
#				t0Errs.append(np.sqrt(pcov[0][0]))
#				taus.append(popt[1])
#				tauErrs.append(np.sqrt(pcov[1][1]))

				t0Run.append(popt[0])
				t0RunErr.append(np.sqrt(pcov[0][0]))
				tauRun.append(popt[1])
				tauRunErr.append(np.sqrt(pcov[1][1]))

				print("%s, %s:\n\tt0 = %f pm %f, tau = %f pm %f\n" % (stiffnessNames[lIndex], modelNames[mIndex], popt[0], np.sqrt(pcov[0][0]), popt[1], np.sqrt(pcov[1][1])))
#logistic(t, t0, tau, L)

			t0Errs.append(np.std(t0Run) / np.sqrt(numRuns) + np.mean(t0RunErr))
			t0s.append(np.mean(t0Run))

			tauErrs.append(np.std(tauRun) / np.sqrt(numRuns) + np.mean(tauRunErr))
			taus.append(np.mean(tauRun))


			mIndex += 1
			
		# Prettify
		plt.figure(traceFig.number)
#		plt.ylim([-0.1,5100])
		plt.xticks([0,10,20,30,40],[0,10,20,30,40])
		plt.yticks([0,1000,2000,3000,4000,5000],[0,1000,2000,3000,4000,5000])

		plt.xlabel(r"Time ($\mu$s)", fontsize=SMALL)
		plt.ylabel("Max Cluster Size", fontsize=SMALL)

		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)

		plt.legend(loc=0, fontsize=SSMALL)
		plt.tight_layout()

		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_ClusterFormationOverTime.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=300)
		plt.clf()

		# Plot
		plt.figure(timeFigT0.number)
		plt.errorbar(range(len(t0s)), t0s, yerr=t0Errs, fmt="-xC%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))

		plt.figure(timeFigTau.number)
		plt.errorbar(range(len(taus)), taus, yerr=tauErrs, fmt="-oC%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))

		lIndex += 1

	# Prettify
	plt.figure(timeFigT0.number)
	plt.xticks(range(len(t0s)), modelNames2[:len(t0s)])
	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Logistic T0 ($\mu$s)", fontsize=SMALL)
	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.legend(loc=0, fontsize=SSMALL)
	plt.tight_layout()

	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_ClusterFormationLogisticT0.png" % (vf)
	plt.savefig(oFname, dpi=300)
	plt.clf()

	plt.figure(timeFigTau.number)
	plt.xticks(range(len(taus)), modelNames2[:len(taus)])
	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Logistic Tau ($\mu$s)", fontsize=SMALL)
	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.legend(loc=0, fontsize=SSMALL)
	plt.tight_layout()

	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_ClusterFormationLogisticTau.png" % (vf)
	plt.savefig(oFname, dpi=300)

	plt.clf()

	vIndex += 1
