import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTopology

def linear(x, m, c):
	return m * x + c

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]

vfs = [0.05,0.07,0.1]

numModels = len(models)
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
flexSymbol = ["x", "o", "^"]
vfs = [0.05]

numRuns = 2

# Figures
totalCoordFig = plt.figure(figsize=(1.693,1.693))
coordFig = plt.figure(figsize=(1.693,1.693))
normCoordFig = plt.figure(figsize=(1.693,1.693))

numBars = monomersPerPolymer + 2
barWidth = 1.0 / numBars

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		totalCoordFinal = np.array([0.0 for nr in range(numModels)])
		totalCoordErrFinal = np.array([0.0 for nr in range(numModels)])
		mIndex = 0
		for model in models:

			totalCoords = np.array([0.0 for nr in range(numRuns)])
			coords = np.array([None for nr in range(numRuns)])
			for nr in range(numRuns):
			
				coord = np.array([0.0 for i in range(monomersPerPolymer)])

				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
				
				tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				topInit = BioNetTopology.BioNetTopology()
				topInit.loadFrame(tpFname, 0)
				
				top = BioNetTopology.BioNetTopology()
				top.loadFinalFrame(tpFname)

				# Calculate the coordination of each protein along the chain
				for i in range(numNodes):
					coord[i % monomersPerPolymer] += top.frame[-1].nCoord[i]
					totalCoords[nr] += top.frame[-1].nCoord[i]

				# Average simulation
				coord /= numPolymers
				coords[nr] = coord
				totalCoords[nr] /= numNodes


			# Average between simulations (for experimental error) 
			coords = coords.transpose()
			coordErr = np.std(coords) / np.sqrt(numRuns)
			coord = np.mean(coords, axis=0)
			c0 = np.array([topInit.frame[-1].nCoord[i] for i in range(monomersPerPolymer)])
			coord -= c0

			tc0 = np.sum(np.array([topInit.frame[-1].nCoord[i] for i in range(monomersPerPolymer)])) / monomersPerPolymer
			totalCoords -= tc0

			totalCoordErr = np.std(totalCoords) / np.sqrt(numRuns)
			totalCoord = np.mean(totalCoords)

			totalCoordFinal[mIndex] = totalCoord
			totalCoordErrFinal[mIndex] = totalCoordErr

#			if (mIndex == 0 and lIndex == 2) or (mIndex == 5 and lIndex == 0):
#				print("Fd: %f pm %f" % (totalCoord, totalCoordErr))

			# Plot
#			xPos = np.array([i for i in range(monomersPerPolymer)]) + (mIndex + 1.5) * barWidth
			xPos = mIndex + (np.array([i for i in range(monomersPerPolymer)]) + 1.5) * barWidth
			
			plt.figure(coordFig.number)
			bar = plt.bar(xPos, coord, barWidth, yerr=coordErr, edgecolor="black", linewidth = 1, color="C%d" % (mIndex), label="%s" % (modelNames2[mIndex]))

			# Rescale and plot
			numSites = np.array([inp.node[i].protein.numSites for i in range(monomersPerPolymer)])
			numSites -= c0

			coordScale = coord / numSites
			coordScaleErr = coordErr / numSites
			
			plt.figure(normCoordFig.number)
			bar = plt.bar(xPos, coordScale, barWidth, yerr=coordScaleErr, edgecolor="black", linewidth = 1, color="C%d" % (mIndex), label="%s" % (modelNames2[mIndex]))

			mIndex += 1
			
		# Plot
		plt.figure(totalCoordFig.number)
		xData = np.array(range(totalCoordFinal.size))
		plt.errorbar(xData, totalCoordFinal, yerr=totalCoordErrFinal, fmt="-" + flexSymbol[lIndex] + "C%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))


		popt, pcov = curve_fit(linear, xData[2:], totalCoordFinal[2:], [1.0,0.0])
		plt.plot(xData[2:], linear(xData[2:], *popt), "--C%d" % (lIndex), markersize=3, linewidth=1)

		m = popt[0]
		mErr = np.sqrt(pcov[0][0])

		print("%s Coordination Gradient: m = %f pm %f" % (stiffnessNames[lIndex], m, mErr))


		# Prettify
		plt.figure(coordFig.number)

		# x ticks at halfway point
#		t = np.array([i for i in range(monomersPerPolymer)]) + 0.5
#		l = [str(i) for i in np.array([i for i in range(monomersPerPolymer)]) + 1]
		t = np.array([i for i in range(numModels)]) + 0.5
		l = [r"M$_%d$" % (i) for i in range(numModels)]
		plt.xticks(t,l)
		
		t = np.array([0.0,1.0,2.0,3.0])
		l = np.array([str(bit) for bit in t])
		plt.yticks(t,l)

		plt.ylim([0.0,3.6])
		plt.xlabel("Model", fontsize=SMALL)
		plt.ylabel("Coordination", fontsize=SMALL)
		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)

		plt.tight_layout()			
		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_NodeCoordination.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=300)
		plt.clf()

		plt.figure(normCoordFig.number)

#		t = np.array([i for i in range(monomersPerPolymer)]) + 0.5
#		l = [str(i) for i in np.array([i for i in range(monomersPerPolymer)]) + 1]
		t = np.array([i for i in range(numModels)]) + 0.5
		l = [r"M$_%d$" % (i) for i in range(numModels)]
		plt.xticks(t,l)
		
		t = np.array([0.0,0.5,1.0])
		l = np.array([str(bit) for bit in t])
		plt.yticks(t,l)

#		plt.xlim()
		plt.ylim([0.0,1.1])
		plt.xlabel("Model", fontsize=SMALL)
		plt.ylabel("Coordination (Norm)", fontsize=SMALL)
		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)

		plt.tight_layout()			
		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_NormNodeCoordination.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=300)
		plt.clf()
		
		lIndex += 1

	# Prettify
	plt.figure(totalCoordFig.number)
	plt.xticks(range(totalCoordFinal.size), modelNames2[:totalCoordFinal.size])
	plt.yticks([0.0,1.0,2.0,3.0],["0.0","1.0","2.0","3.0"])
	plt.ylim([-0.1,3.1])
	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Coordination", fontsize=SMALL)

	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.legend(loc=0, fontsize=SSMALL)
	plt.tight_layout()

	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_FinalCoordination.png" % (vf)
	plt.savefig(oFname, dpi=300)
	plt.clf()
	vIndex += 1
