import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTopology, BioNetMeasurement

def expFit(t, tau, y0, yf):

	return (yf - y0) * (1 - np.exp(-t / tau)) + y0

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]

vfs = [0.05,0.07,0.1]

modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
vfs = [0.05]

numRuns = 2

# Figures
traceFig = plt.figure(figsize=(1.693,1.693))

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		linkerDl = lOn2R * 2 * radius
		linkerK = kbt / np.power(linkerDl, 2.0)
		Lc = (monomersPerPolymer - 1) * (2 * radius + linkerL + linkerDl) + 2 * radius
		
		mIndex = 0
		for model in models:

			# Time trace (every 50 frames)
			stepsize = 50

			for nr in range(numRuns):
			
				time = []
				coordTrace = []

				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
	
				mFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bnms" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				meas = BioNetMeasurement.BioNetMeasurement()
				meas.load(mFname)

				tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				topInit = BioNetTopology.BioNetTopology()
				topInit.loadFrame(tpFname, 0)
				
				time = meas.time[::stepsize]

				top = BioNetTopology.BioNetTopology()
				top.load(tpFname, rate=stepsize)

				time = []
				coordTrace = []

				# Get corrected coordination
				for frameIndex in range(top.numFrames):			

					time.append(top.frame[frameIndex].time)

					coord = 0
					for i in range(numNodes):
						coord += top.frame[frameIndex].nCoord[i]

					coord /= numNodes

					c0 = np.sum(np.array([topInit.frame[-1].nCoord[i] for i in range(monomersPerPolymer)])) / monomersPerPolymer
					coord -= c0

					coordTrace.append(coord)

				# Plot
				time = np.array(time) / 1000.0
				plt.figure(traceFig.number)
				if nr == 0:
					plt.plot(time, coordTrace, "-xC%d" % (mIndex), markersize = 3, linewidth = 1, label=r"M$_%d$" % (mIndex))
				else:
					plt.plot(time, coordTrace, "-xC%d" % (mIndex), markersize = 3, linewidth = 1)

			mIndex += 1
			
		# Prettify
		plt.figure(traceFig.number)

		t = np.array([0,10,20,30,40])
		l = np.array([str(int(bit)) for bit in t])
		plt.xticks(t,l)

		t = np.array([0.0,1.0,2.0,3.0])
		l = np.array([str(bit) for bit in t])
		plt.yticks(t,l)

		plt.ylim([-0.1,3.1])

		plt.xlabel(r"Time ($\mu$s)", fontsize=SMALL)
		plt.ylabel("Coordination", fontsize=SMALL)

		for tick in plt.gca().xaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)
		for tick in plt.gca().yaxis.get_major_ticks():
			tick.label.set_fontsize(SMALL)


		plt.legend(loc=0, fontsize=SSMALL)
		plt.tight_layout()

		oFname = "../results/AnalyticalPolymerGel_%d%sPolymer_vf%.3f_Kinetic_CoordinationOverTime.png" % (lIndex, stiffnessNames[lIndex], vf)
		plt.savefig(oFname, dpi=100)
		plt.clf()

		lIndex += 1

	vIndex += 1
