import sys, os
import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit

import BioNetInput, BioNetTopology

def linear(x, m, c):
	return m * x + c

SSMALL = 4
SMALL = 8
MEDIUM = 10
LARGE = 12

# System data
numPolymers = 1000
monomersPerPolymer = 5
numNodes = numPolymers * monomersPerPolymer
radius = 2.5
drag = 6 * np.pi * radius
kbt = 4.11

linkerL = 0

crossL = 0.15
crossK = kbt / np.power(0.1, 2)

stericK = 10000

# Site stuff (7 types of protein in the models)
modelProteins = []
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([1.0,0.0,0.0])])
modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([1.0,0.0,0.0])])

modelProteins.append([np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0]), np.array([1.0,0.0,0.0])])


# Variables
models = [[0,0,1,0,0], [0,0,2,0,0], [0,1,2,1,0], [0,2,2,2,0], [1,2,2,2,1], [2,2,2,2,2]]
modelNames = ["GC%d-A" % (i) for i in range(len(models))]

lOn2Rs = [0.132,0.061,0.023]	# These give Lp / Lc = [0.5,1.0,1.5] as calculated by plotPersistenceLengths.py
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]

vfs = [0.05,0.07,0.1]

numModels = len(models)
modelNames = ["GC%d-A" % (i) for i in range(len(models))]
modelNames2 = [r"M$_%d$" % (i) for i in range(len(models))]
stiffnessNames = ["Flexible", "SemiFlexible", "Rigid"]
vfs = [0.05]

numRuns = 2

# Figures
coordFig = plt.figure(figsize=(1.693,1.693))

vIndex = 0		
for vf in vfs:

	lIndex = 0
	for lOn2R in lOn2Rs:

		intersTrace = []
		intersErrTrace = []

		intrasTrace = []
		intrasErrTrace = []

		mIndex = 0
		for model in models:

			intersFrac = []
			intrasFrac = []
			for nr in range(numRuns):
			

				# Read
				iFname = "../input/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.xml" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
				inp = BioNetInput.BioNetInput()
				inp.load(iFname)
				
				tpFname = "../output/AnalyticalPolymerGel_Model%s_%sPolymer_vf%.3f_Kinetic_Run%d.bntop" % (modelNames[mIndex], stiffnessNames[lIndex], vf, nr)
#				topInit = BioNetTopology.BioNetTopology()
#				topInit.loadFrame(tpFname, 0)
				
				top = BioNetTopology.BioNetTopology()
				top.loadFinalFrame(tpFname)

				# Calculate the pair coordination of each bond along the chain
				numInter = 0
				numIntra = 0
				for c in top.frame[0].cConnections:
					nIndex = [c[0][1], c[1][1]]
					
					pIndex = [int(n / 5) for n in nIndex]
					if pIndex[0] == pIndex[1]:
						numIntra += 1
					else:
						numInter += 1

				# Remove linkers from the count
				numIntra -= numPolymers * 4

				numCC = numIntra + numInter
				intrasFrac.append(numIntra / numCC)
				intersFrac.append(numInter / numCC)

			# Stats
			interFracErr = np.std(intersFrac) / np.sqrt(numRuns)
			interFrac = np.mean(intersFrac)

			intraFracErr = np.std(intrasFrac) / np.sqrt(numRuns)
			intraFrac = np.mean(intrasFrac)

			# Array filling
			intersTrace.append(interFrac)
			intersErrTrace.append(interFracErr)

			intrasTrace.append(intraFrac)
			intrasErrTrace.append(intraFracErr)

			mIndex += 1
	
		# Plot
		plt.figure(coordFig.number)
		plt.errorbar(range(numModels), intersTrace, yerr=intersErrTrace, fmt="-xC%d" % (lIndex), markersize=3, linewidth=1, label="%s" % (stiffnessNames[lIndex]))
		plt.errorbar(range(numModels), intrasTrace, yerr=intrasErrTrace, fmt="-oC%d" % (lIndex), markersize=3, linewidth=1)		
		lIndex += 1

	# Prettify
	plt.figure(coordFig.number)
	plt.xticks(range(numModels), modelNames2[:numModels])
	plt.yticks([0.0,0.25,0.5,0.75,1],["0.0","0.25","0.5","0.75","1.0"])
	plt.ylim([-0.1,1.1])
	plt.xlabel("Model", fontsize=SMALL)
	plt.ylabel("Pair Coordination", fontsize=SMALL)

	for tick in plt.gca().xaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)
	for tick in plt.gca().yaxis.get_major_ticks():
		tick.label.set_fontsize(SMALL)

	plt.legend(loc=0, fontsize=SSMALL)
	plt.tight_layout()

	oFname = "../results/AnalyticalPolymerGel_vf%.3f_Kinetic_PairCoordination.png" % (vf)
	plt.savefig(oFname, dpi=300)
	plt.clf()

	vIndex += 1
